SorhaFrameDrag = LibStub("AceAddon-3.0"):NewAddon("SorhaFrameDrag", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0", "AceHook-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("SorhaFrameDrag")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local MODNAME = "SorhaFrameDrag";

BINDING_HEADER_SORHAFRAMEDRAG = "Sorha Frame Drag";
BINDING_NAME_TOGGLE_SORHAFRAMEDRAG = L["Toggle Minion"]


local tblLazyLoadedFramesToBeLoaded = {}
local tblSetupFrames = {};

local lazyLoadMainFrames = false;
local lazyEnableMainFrames = false;
local lazyDisableMainFrames = false;


local tblLazyLoadedFrames = {
	Blizzard_PetJournal = {
		Loaded = false,
		Frame = "PetJournalParent",
		InCombat = false,
		Enabled = false,
	},
	Blizzard_TalentUI = {
		Loaded = false,
		Frame = "PlayerTalentFrame",
		InCombat = true,
		Enabled = false,
	},
	Blizzard_GuildUI = {
		Loaded = false,
		Frame = "GuildFrame",
		DisableMouseSubframes = {
			"TitleMouseover"
		},	
		InCombat = true,
		Enabled = false,
	},
	Blizzard_StoreUI = {
		Loaded = false,
		Frame = "StoreFrame",
		InCombat = false,
		Enabled = false,
	},
	Blizzard_EncounterJournal = {
		Loaded = false,
		Frame = "EncounterJournal",
		InCombat = true,
		Enabled = false,
	},
	Blizzard_AchievementUI = {
		Loaded = false,
		Frame = "AchievementFrame",
		DisableMouseFrames = {
			"AchievementFrameHeader"
		},	
		InCombat = true,
		Enabled = false,
	},
	Blizzard_BindingUI = {
		Loaded = false,
		Frame = "KeyBindingFrame",
		InCombat = true,
		Enabled = false,
	},
	Blizzard_TradeSkillUI = {
		Loaded = false,
		Frame = "TradeSkillFrame",
		InCombat = true,
		Enabled = false,
	},
	Blizzard_MacroUI = {
		Loaded = false,
		Frame = "MacroFrame",
		InCombat = true,
		Enabled = false,
	},
}



local tblDisableMouseFrames = {
	"ReputationFrame",
}


--Defaults
local db
local defaults = {
	profile = {
		DragEnabled = false,
	},
}

local options = nil
local moduleOptions = {}
local function getOptions()
	if not options then
		options = {
			name = "SorhaFrameDrag ".. "r2",
			type = "group",
			handler = SorhaFrameDrag,
			args = {
				General = {
					name = L["General Options"],
					type = "group",
					order = 1,
					args = {
						enabled = {
							order = 1,
							type = "toggle",
							name = L["Enable Frame Drag"],
							get = function() return db.DragEnabled end,
							set = function(info, value) 
								db.DragEnabled = value;
								if (value == true) then
									SorhaFrameDrag:EnableFrameDrag();
								else
									SorhaFrameDrag:DisableFrameDrag();
								end
							end,
						},	
					}
				}
			}
		}
		for k,v in pairs(moduleOptions) do
			options.args[k] = (type(v) == "function") and v() or v
		end
	end

	return options
end

local function hook_UIParentLoadAddOn(name)
	if (SorhaFrameDrag:IsEnabled() == false) then
		return;
	end
	if (tblLazyLoadedFrames[name] == nil) then
		print(name .. " NYI");
		return
	end
	if (InCombatLockdown() and tblLazyLoadedFrames[name].InCombat == false) then
		SorhaFrameDrag:RegisterEvent('PLAYER_REGEN_ENABLED');
		tinsert(tblLazyLoadedFramesToBeLoaded, name)
		return
	end
	SorhaFrameDrag:SetupLazyLoadFrame(name);
end

local function HandleChatCommand()
	InterfaceOptionsFrame_OpenToCategory(SorhaFrameDrag.optionsFrames.Profiles)
	InterfaceOptionsFrame_OpenToCategory(SorhaFrameDrag.optionsFrames.SorhaFrameDrag)
	InterfaceOptionsFrame:Raise()
end

function SorhaFrameDrag:OnInitialize() 
	self.db = LibStub("AceDB-3.0"):New("SorhaFrameDragDB", defaults, true)
	db = self.db.profile

	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")	
	self:SetupOptions()

	hooksecurefunc("UIParentLoadAddOn", hook_UIParentLoadAddOn)

	self:InitTempDraggableFrames();
end

function SorhaFrameDrag:OnEnable()	
	if (db.DragEnabled == true) then
		self:EnableFrameDrag();
	else
		self:DisableFrameDrag();
	end
end

function SorhaFrameDrag:PLAYER_REGEN_ENABLED(...)
	for key, name in pairs(tblLazyLoadedFramesToBeLoaded) do
		self:SetupLazyLoadFrame(name);
		tblLazyLoadedFramesToBeLoaded[key] = nil;
	end

	if (lazyLoadMainFrames == true) then
		lazyLoadMainFrames = false;
		self:InitTempDraggableFrames();
	end
	if (lazyEnableMainFrames == true) then
		lazyEnableMainFrames = false;
		self:EnableFrameDrag();
	end
	if (lazyDisableMainFrames == true) then
		lazyDisableMainFrames = false;
		self:DisableFrameDrag();
	end

	self:UnregisterEvent("PLAYER_REGEN_ENABLED")
end


function SorhaFrameDrag:SetupOptions() --Creates options. Registers chat commands
	self.optionsFrames = {}
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("SorhaFrameDrag", getOptions)
	self.optionsFrames.SorhaFrameDrag = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SorhaFrameDrag", nil, nil, "General")
	
	self:RegisterModuleOptions("Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db), "Profiles")
	
	LibStub("AceConsole-3.0"):RegisterChatCommand("sfd", HandleChatCommand)
	LibStub("AceConsole-3.0"):RegisterChatCommand("sorhaframedrag", HandleChatCommand)
end

function SorhaFrameDrag:RegisterModuleOptions(name, optionTbl, displayName)
	moduleOptions[name] = optionTbl	
	self.optionsFrames[name] = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SorhaFrameDrag", displayName or name, "SorhaFrameDrag", name)
end

function SorhaFrameDrag:RefreshConfig()
	db = self.db.profile

	if (db.DragEnabled == true) then
		self:EnableFrameDrag();
	else
		self:DisableFrameDrag();
	end
end



--Frame setups
function SorhaFrameDrag:InitTempDraggableFrames()
	if (InCombatLockdown()) then
		lazyLoadMainFrames = true;
		self:RegisterEvent("PLAYER_REGEN_ENABLED");
		return
	end

	for frameName,info in pairs(UIPanelWindows) do
		if (info.area ~= "full") then
			local frame = _G[frameName]
			if (frame) then
				local titleRegion = frame:GetTitleRegion()
				if (not titleRegion) then
					titleRegion = frame:CreateTitleRegion()
				end
				titleRegion:SetAllPoints(frame)
			end
			tinsert(tblSetupFrames, frame);
		end
	end
end

function SorhaFrameDrag:SetupLazyLoadFrame(name)
	if (tblLazyLoadedFrames[name].Loaded == true) then
		return
	end

	local frame = _G[tblLazyLoadedFrames[name].Frame];
	if (frame ~= nil) then
		tblLazyLoadedFrames[name].Loaded = true;

		local titleRegion = frame:GetTitleRegion()
		if (not titleRegion) then
			titleRegion = frame:CreateTitleRegion()
		end

		titleRegion:SetAllPoints(frame)

	if (db.DragEnabled == true) then
		self:EnableLazyLoadFrame(name);
	end
		
	end
end


--Frame Enables
function SorhaFrameDrag:EnableFrameDrag()
	if (InCombatLockdown()) then
		lazyEnableMainFrames = true;
		self:RegisterEvent("PLAYER_REGEN_ENABLED");
		return
	end

	for i, frame in ipairs(tblSetupFrames) do
		frame:EnableMouse(1)
	end

	for i,frameName in ipairs(tblDisableMouseFrames) do
		local frame = _G[frameName]
		if (frame) then
			frame:EnableMouse(nil)
		end
	end

	for name, tbl in ipairs(tblLazyLoadedFrames) do
		if (tbl.Loaded == true and tbl.Enabled == false) then
			self:EnableLazyLoadFrame(name);
		end
	end
end

function SorhaFrameDrag:DisableFrameDrag()
	if (InCombatLockdown()) then
		lazyDisableMainFrames = true;
		self:RegisterEvent("PLAYER_REGEN_ENABLED");
		return
	end

	for i, frame in ipairs(tblSetupFrames) do
		frame:EnableMouse(nil)
	end

	for i,frameName in ipairs(tblDisableMouseFrames) do
		local frame = _G[frameName]
		if (frame) then
			frame:EnableMouse(1)
		end
	end

	for name, tbl in ipairs(tblLazyLoadedFrames) do
		if (tbl.Loaded == true and tbl.Enabled == true) then
			self:DisableLazyLoadFrame(name);
		end
	end
end


function SorhaFrameDrag:EnableLazyLoadFrame(name)
	local frame = _G[tblLazyLoadedFrames[name].Frame];
	if (frame ~= nil) then
		frame:EnableMouse(1)

		--Kill mouse on subframes
		if (tblLazyLoadedFrames[name].DisableMouseSubframes) then
			for key, value in pairs(tblLazyLoadedFrames[name].DisableMouseSubframes) do
				local frameDisable = frame[value];
				if (frameDisable) then
					frameDisable:EnableMouse(nil)
				end
			end
		end

		--Kill mouse on other frames
		if (tblLazyLoadedFrames[name].DisableMouseFrames) then
			for key, value in pairs(tblLazyLoadedFrames[name].DisableMouseFrames) do
				local frameDisable = _G[value];
				if (frameDisable) then
					frameDisable:EnableMouse(nil)
				end
			end
		end
	end
end

function SorhaFrameDrag:DisableLazyLoadFrame(name)
	local frame = _G[tblLazyLoadedFrames[name].Frame];
	if (frame ~= nil) then
		frame:EnableMouse(nil)

		--Kill mouse on subframes
		if (tblLazyLoadedFrames[name].DisableMouseSubframes) then
			for key, value in pairs(tblLazyLoadedFrames[name].DisableMouseSubframes) do
				local frameDisable = frame[value];
				if (frameDisable) then
					frameDisable:EnableMouse(1)
				end
			end
		end

		--Kill mouse on other frames
		if (tblLazyLoadedFrames[name].DisableMouseFrames) then
			for key, value in pairs(tblLazyLoadedFrames[name].DisableMouseFrames) do
				local frameDisable = _G[value];
				if (frameDisable) then
					frameDisable:EnableMouse(1)
				end
			end
		end
	end
end


--Outside Methods
function SorhaFrameDrag:InitCustomFrame(name, frameName)
	local tblNewFrame = {
		Loaded = false,
		Frame = frameName,
		InCombat = false,
		Enabled = false,
	}
	tblLazyLoadedFrames[name]= tblNewFrame;
	self:SetupLazyLoadFrame(name)
end


function SorhaFrameDrag:ToggleFrameDrag()
	db.DragEnabled = not db.DragEnabled ;
	if (db.DragEnabled == true) then
		self:EnableFrameDrag();
	else
		self:DisableFrameDrag();
	end
end
